<?php
/**
 * Copyright 2013 Drexel University
 */

require_once ((__dir__) . "/irods/RODSKeyValPair.class.php");
require_once ((__dir__) . "/irods/ProdsRule.class.php");
require_once ((__dir__) . "/irods/RODSMeta.class.php");
require_once ((__dir__) . "/irods/ProdsTicket.class.php");
require_once ((__dir__) . "/irods/packet/RODSPacket.class.php");
require_once ((__dir__) . "/irods/packet/RP_ModAVUMetadataInp.class.php");
require_once ((__dir__) . "/irods/RODSException.class.php");
require_once ((__dir__) . "/irods/packet/RP_DataObjCopyInp.class.php");
require_once ((__dir__) . "/irods/packet/RP_CollOprStat.class.php");
require_once ((__dir__) . "/irods/packet/RP_dataObjCloseInp.class.php");
require_once ((__dir__) . "/irods/packet/RP_dataObjWriteInp.class.php");
require_once ((__dir__) . "/irods/packet/RP_CollInp.class.php");
require_once ((__dir__) . "/irods/packet/RP_DataObjInp.class.php");
require_once ((__dir__) . "/irods/packet/RP_SqlResult.class.php");
require_once ((__dir__) . "/irods/packet/RP_GenQueryOut.class.php");
require_once ((__dir__) . "/irods/packet/RP_GenQueryInp.class.php");
require_once ((__dir__) . "/irods/packet/RP_InxValPair.class.php");
require_once ((__dir__) . "/irods/packet/RP_KeyValPair.class.php");
require_once ((__dir__) . "/irods/packet/RP_InxIvalPair.class.php");
require_once ((__dir__) . "/irods/RODSQueryCondition.class.php");
require_once ((__dir__) . "/irods/packet/RP_authResponseInp.class.php");
require_once ((__dir__) . "/irods/packet/RP_authRequestOut.class.php");
require_once ((__dir__) . "/irods/packet/RP_Version.class.php");
require_once ((__dir__) . "/irods/packet/RP_MsgHeader.class.php");
require_once ((__dir__) . "/irods/packet/RP_MsParam.class.php");
require_once ((__dir__) . "/irods/packet/RP_fileLseekInp.class.php");
require_once ((__dir__) . "/irods/packet/RP_fileLseekOut.class.php");
require_once ((__dir__) . "/irods/packet/RP_dataObjReadInp.class.php");
require_once ((__dir__) . "/irods/packet/RP_ticketAdminInp.class.php");

require_once ((__dir__) . "/irods/packet/RP_STR.class.php");
require_once ((__dir__) . "/irods/packet/RP_BinBytesBuf.class.php");
require_once ((__dir__) . "/irods/packet/RP_MsParamArray.class.php");
require_once ((__dir__) . "/irods/packet/RP_RHostAddr.class.php");
require_once ((__dir__) . "/irods/packet/RP_ExecMyRuleInp.class.php");

require_once ((__dir__) . "/irods/RODSMessage.class.php");
require_once ((__dir__) . "/irods/packet/RP_StartupPack.class.php");
require_once ((__dir__) . "/irods/RODSFileStats.class.php");
require_once ((__dir__) . "/irods/RODSGenQueResults.class.php");
require_once ((__dir__) . "/irods/RODSGenQueConds.class.php");
require_once ((__dir__) . "/irods/RODSGenQueSelFlds.class.php");
require_once ((__dir__) . "/irods/RODSConn.class.php");
require_once ((__dir__) . "/irods/RODSConnManager.class.php");
require_once ((__dir__) . "/irods/ProdsPath.class.php");
require_once ((__dir__) . "/irods/ProdsFile.class.php");
require_once ((__dir__) . "/irods/ProdsDir.class.php");
require_once ((__dir__) . "/irods/RODSAccount.class.php");

function getCustomAccount() {
    global $wgUser;

    if ($wgUser->mId == 0)
        return null;

    $dbr = wfGetDB(DB_SLAVE);
    return $dbr->selectRow(
                    array("irods_users"), array("irods_user_name, irods_user_password, irods_user_resource, irods_user_zone"), array("wiki_user_id" => $wgUser->mId, "irods_custom_account_active" => 1)
    );
}

function getAccountReadFile($path, $ticket = '') {
    $custom = getAccount(false, $ticket);
    if ($ticket != '') {
        //TODO: Validate ticket
        return $custom;
    }

    $file = new ProdsFile($custom, $path);
    $acl = $file->getACL();

    foreach ($acl as $access) {
        if ($access->COL_USER_NAME == $custom->user) {
            return $custom;
        }
    }

    $default = getAccount(true, $ticket);
    foreach ($acl as $access) {
        if ($access->COL_USER_NAME == $default->user) {
            return $default;
        }
    }

    return null;
}

function getAccountWriteFile($path, $ticket = '') {
    $custom = getAccount(false, $ticket);
    if ($ticket != '') {
        //TODO: Validate ticket
        return $custom;
    }

    $file = new ProdsFile($custom, $path);
    $acl = $file->getACL();


    foreach ($acl as $access) {
        if ($access->COL_USER_NAME == $custom->user) {
            if ($access->COL_DATA_ACCESS_NAME == 'own' || $access->COL_DATA_ACCESS_NAME == 'modify object') {
                return $custom;
            }
        }
    }

    $default = getAccount(true, $ticket);
    foreach ($acl as $access) {
        if ($access->COL_USER_NAME == $default->user) {
            if ($access->COL_DATA_ACCESS_NAME == 'own' || $access->COL_DATA_ACCESS_NAME == 'modify object') {
                return $default;
            }
        }
    }

    return null;
}

function getAccountWriteCollection($path, $ticket = '') {

    $custom = getAccount(false, $ticket);
    if ($ticket != '') {
        //TODO: Validate ticket
        return $custom;
    }
    $dir = new ProdsDir($custom, $path);
    if ($dir->exists())
        $acl = $dir->getACL();
    else
        return null;

    $users = $acl['COL_USERS'];

    foreach ($users as $access) {
        if ($access->COL_USER_NAME == $custom->user) {
            if ($access->COL_DATA_ACCESS_NAME == 'own' || $access->COL_DATA_ACCESS_NAME == 'modify object') {
                return $custom;
            }
        }
    }

    $default = getAccount(true, $ticket);
    foreach ($users as $access) {
        if ($access->COL_USER_NAME == $default->user) {
            if ($access->COL_DATA_ACCESS_NAME == 'own' || $access->COL_DATA_ACCESS_NAME == 'modify object') {
                return $default;
            }
        }
    }

    return null;
}

// creates a global irods connection that could be used by all functions
function getAccount($forceDefault = false, $ticket = '') {
    global $configIrodsHost, $configIrodsPort, $configIrodsUser, $configIrodsPass,
    $configIrodsResource, $configIrodsZone;

    if ($forceDefault) {
        return new RODSAccount(
                $configIrodsHost, $configIrodsPort, $configIrodsUser, $configIrodsPass, $configIrodsZone, $configIrodsResource, $ticket
        );
    }

    $customInfo = getCustomAccount();
    if ($customInfo != null) {
        return new RODSAccount(
                $configIrodsHost, $configIrodsPort, $customInfo->irods_user_name, $customInfo->irods_user_password, $customInfo->irods_user_zone, $customInfo->irods_user_resource, $ticket
        );
    } else {
        return new RODSAccount(
                $configIrodsHost, $configIrodsPort, $configIrodsUser, $configIrodsPass, $configIrodsZone, $configIrodsResource, $ticket
        );
    }
}

function get_user_rights_on_irods_file($path, $user) {
    $myfile = new ProdsFile(getAccount(true), $path); // creates irods file object

    $rights = $myfile->getACL();

    for ($i = 0; $i < sizeof($rights); $i++) {
        if ($rights[$i]->COL_USER_NAME == $user)
            return $rights[$i];
    }
    return false;
}

function make_directory($path, $dir_name) {
    if (existCollection($path . $dir_name)) {
        return getAccountWriteCollection($path . $dir_name);
    } else {
        $account = make_directory(dirname($path) . '/', basename($path));

        $dir = new ProdsDir($account, $path);
        $newdir = new ProdsDir($account, $path . $dir_name);


        try {
            if (!$newdir->exists())
                $mydir = $dir->mkdir($dir_name); // creates dir
        } catch (Exception $e) {
            if ($e->getCode() == -818000)
                die("User not allowed to create iRODS collections.");
        }
        return $account;
    }
}

function uploadFileOnIrods(&$src_path, $dest_path) {
    $account = getAccountWriteCollection(dirname($dest_path));
    // set running time for PHP to 0 = no limit
    set_time_limit(0);

    $file_to_be_uploaded = file_get_contents($src_path); // gets the content of the file and stores it in memory

    try {
        $myfile = new ProdsFile($account, $dest_path); // creates irods file object
        //read and print out the file
        $myfile->open("w+", $account->default_resc); // creates an empty file on irods server

        $myfile->write($file_to_be_uploaded); // fills the empty file with the content stored in memory on irods server
        //close the file pointer
        $myfile->close();
    } catch (Exception $e) {
        if ($e->getCode() == -818000)
            die("User not allowed to upload iRODS file at this location.");
        else
            die("An error occured when trying to upload the file at the specified path.");
        return false;
    }

    $myfile_check = new ProdsFile($account, $dest_path); // creates irods file object to check if it was successfully written
    $fileInfo = $myfile_check->getReplInfo();

    if ($fileInfo[0]['size'] == filesize($src_path))
        return true; // in case the file was completely uploaded

    return false;
}

function getIrodsMetadata($filepath) {
    $account = getAccount(true);
    $myfile = new ProdsFile($account, $filepath);
    $metadata = $myfile->getReplInfo();
    $avus = $myfile->getMeta();

    for ($i = 0; $i < sizeof($avus); $i++) {
        $avus_array [$i][0] = $avus_array [$i]['name'] = $avus[$i]->name;
        $avus_array [$i][1] = $avus_array [$i]['value'] = $avus[$i]->value;
        $avus_array [$i][2] = $avus_array [$i]['units'] = $avus[$i]->units;
    }


    if (!isset($metadata[0]))
        $metadata[0] = array();
    $metadata = array_merge(
            array('filename' => basename($filepath)), array('AVUs' => $avus_array), // for AVUs
            $metadata[0]
    );
    return $metadata;
}

function updateAVUs($filepath, $attr_old, $val_old, $unit_old, $attr_new, $val_new, $unit_new, $ticket = '') {
    try {
        if (existsAVU($filepath, $attr_new, $val_new, $unit_new))
            addAVU($filepath, $attr_new, $val_new, $unit_new);
        else {
            $account = getAccountWriteFile($filepath, $ticket);
            if ($account == null)
                die("User not allowed to update the metadata of this file object.");
            $myfile = new ProdsFile($account, $filepath);

            $meta_old = new RODSMeta($attr_old, $val_old, $unit_old);
            $meta_new = new RODSMeta($attr_new, $val_new, $unit_new);

            $myfile->updateMeta($meta_old, $meta_new);
        }
    } catch (Exception $e) {
        if ($e->getCode() == -818000)
            die("User not allowed to update the metadata of this file object.");
        else
            print_r($e);
    }
}

function existsAVU($filepath, $attr, $val, $unit) {
    $metadata = getIrodsMetadata($filepath);
    for ($i = 0; $i < sizeof($metadata['AVUs']); $i++)
        if ($metadata['AVUs'][$i]['name'] == $attr &&
                $metadata['AVUs'][$i]['value'] == $val &&
                $metadata['AVUs'][$i]['units'] == $unit
        )
            return true;

    return false;
}

function addAVU($filepath, $attr_new, $val_new, $unit_new, $ticket = '') {
    try {
        if (existsAVU($filepath, $attr_new, $val_new, $unit_new))
            return 0;
        else {
            $account = getAccountWriteFile($filepath, $ticket);
            $myfile = new ProdsFile($account, $filepath);

            $meta_new = new RODSMeta($attr_new, $val_new, $unit_new);

            $myfile->addMeta($meta_new);
        }
    } catch (Exception $e) {
        if ($e->getCode() == -818000)
            die("User not allowed to add metadata on this iRODS file.");
        else
            print_r($e);
    }
}

function deleteAVU($filepath, $attr, $val, $unit, $ticket = '') {
    try {
        if (existsAVU($filepath, $attr, $val, $unit)) {
            $account = getAccountWriteFile($filepath, $ticket);
            $myfile = new ProdsFile($account, $filepath);
            $meta = new RODSMeta($attr, $val, $unit);
            $myfile->rmMeta($meta);
        }
    } catch (Exception $e) {
        if ($e->getCode() == -818000)
            die("User not allowed to delete metadata of this iRODS file.");
        else
            print_r($e);
    }
}

function add_irods_avus_to_content_files($filepath, &$text, $art_id) {
    global $wgDBprefix;

    if (existFile($filepath)) {

        // prepare AVU data to be written
        // files that the article links to
        $local_files = local_files_post($text);
        $irods_files = irods_files_post($text);

        // other links in the article
        $internal_links = internal_links_post($art_id);
        $external_links = external_links_post($art_id);

        // get the category 
        $cat = array();
        $sql = "SELECT cl_to FROM " . $wgDBprefix . "categorylinks WHERE cl_from='" . $art_id . "' AND cl_type LIKE 'page'";

        $res = mysql_query($sql);
        while ($row = mysql_fetch_row($res)) {
            $cat [] = $row[0];
        }

        // get existing irods AVUs

        $existing_avus = getIrodsMetadata($filepath);
        for ($i = 0; $i < sizeof($existing_avus['AVUs']); $i++) {
            switch (substr($existing_avus['AVUs'][$i]->name, 0, strpos($existing_avus['AVUs'][$i]->name, ':'))) {
                case "Local file":
                    deleteAVU($filepath, $existing_avus['AVUs'][$i]->name, $existing_avus['AVUs'][$i]->value, $existing_avus['AVUs'][$i]->units);
                    break;
                case "Irods File":
                    deleteAVU($filepath, $existing_avus['AVUs'][$i]->name, $existing_avus['AVUs'][$i]->value, $existing_avus['AVUs'][$i]->units);
                    break;
                case "Internal Link":
                    deleteAVU($filepath, $existing_avus['AVUs'][$i]->name, $existing_avus['AVUs'][$i]->value, $existing_avus['AVUs'][$i]->units);
                    break;
                case "External Link":
                    deleteAVU($filepath, $existing_avus['AVUs'][$i]->name, $existing_avus['AVUs'][$i]->value, $existing_avus['AVUs'][$i]->units);
                    break;
                case "Category":
                    deleteAVU($filepath, $existing_avus['AVUs'][$i]->name, $existing_avus['AVUs'][$i]->value, $existing_avus['AVUs'][$i]->units);
                    break;
            }
        }

        // add AVUs to the file
        for ($i = 0; $i < sizeof($local_files); $i++)
            addAVU($filepath, 'Local file: ' . $i, $local_files[$i], "string");
        for ($i = 0; $i < sizeof($irods_files); $i++)
            addAVU($filepath, 'Irods File: ' . $i, $irods_files[$i], "string");
        for ($i = 0; $i < sizeof($internal_links); $i++)
            addAVU($filepath, 'Internal Link: ' . $i, $internal_links[$i], "string");
        for ($i = 0; $i < sizeof($external_links); $i++)
            addAVU($filepath, 'External Link: ' . $i, $external_links[$i], "string");
        if (sizeof($cat) > 1)
            for ($i = 0; $i < sizeof($external_links); $i++)
                if ($cat[$i] != "Pages_with_broken_file_links")
                    addAVU($filepath, 'Category: ' . $i, $cat[$i], "string");
                elseif (sizeof($cat) == 1)
                    if ($cat[$i] != "Pages_with_broken_file_links")
                        addAVU($filepath, 'Category', $cat[0], "string");
    }
}

function deleteFile($dest_path) {

    $account = getAccount();
    $myfile = new ProdsFile($account, $dest_path); // creates irods file object

    if ($myfile->exists()) {
        $account_rights = getAccountWriteFile($dest_path);
        if ($account != $account_rights) {
            $account = $account_rights;
            $myfile = new ProdsFile($account, $dest_path); // creates irods file object
        } elseif ($account_rights == null)
            return false;
    }



    try {
        if ($myfile->exists())
            $myfile->unlink($account->default_resc); // delete command

        $myfile->close();
    } catch (Exception $e) {
        if ($e->getCode() == -818000)
            die("User not allowed to delete this iRODS file.");
        else
            print_r($e);
    }
}

function writeFile($dest_path, &$content) {
    $account = getAccountWriteCollection(dirname($dest_path));

    try {
        $myfile = new ProdsFile($account, $dest_path); // creates irods file object
        //read and print out the file
        $myfile->open("w+", $account->default_resc); // creates an empty file on irods server

        $myfile->write($content); // fills the empty file with the content stored in memory on irods server
        //close the file pointer
        $myfile->close();
    } catch (Exception $e) {
        if ($e->getCode() == -818000)
            die("User not allowed to write iRODS files in this collection.");
        else
            print_r($e);
    }
}

function renameFile($file, $new_name, $customUser = true) {
    if ($customUser)
        $account = getAccountWriteFile($file);
    else
        $account = getAccount(true);

    try {
        $myfile = new ProdsFile($account, $file); // creates irods file object
        //read and print out the file
        if ($myfile->exists())
            $myfile->rename($new_name); // creates an empty file on irods server



            
//close the file pointer    
        $myfile->close();
    } catch (Exception $e) {
        if ($e->getCode() == -818000)
            die("User not allowed to rename this iRODS file.");
        else
            print_r($e);
    }

    return true;
}

function existFile($file) {
    $account = getAccount(true);

    // creates irods file object
    $myfile = new ProdsFile($account, $file);


    // checks if the file exists and returns the result
    return $myfile->exists();
}

function existCollection($path) {
    $account = getAccount(true);
    $newdir = new ProdsDir($account, $path);
    return $newdir->exists();
}

function get_irods_child_files($path) {
    $account = getAccount(true);
    $dir = new ProdsDir($account, $path);
    return $dir->getChildFiles();
}

function get_irods_path_info($input_filename, &$full_file_path, &$filename, &$format) {
    global $configIrodsRootPath, $additional_path;

    $fileinfo = pathinfo($input_filename);
    $format = $fileinfo['extension'];
    if (substr($input_filename, 0, 1) == '/') {
        $original_filename = strip_tags($input_filename);
        $filename = $fileinfo['basename'];
        $full_file_path = $original_filename;
    } else {
        $filename = filename_to_legal_characters(strip_tags(str_replace('\\', ' ', $input_filename)));
        $additional_path = dirname(get_wiki_file_path($filename));
        $full_file_path = $configIrodsRootPath . $additional_path . '/' . $filename;
    }
}

function local_files_post($text) {
    $matches = array();
    $files_to_upload = array();

    // find all the files that are linked to in this article
    // Old media regular expression:   /\[\[[mM][eE][dD][iI][aA]:(?P<filename>[^|\]]*)[^\]]*?/
    if (preg_match_all("/\[\[[mM][eE][dD][iI][aA]:(?P<filename>[^\|!@\$&\*\?:\[\]\"<>'`=\{\}\\/,;]*)[^\]]*\]\]/", $text, $matches)) {
        for ($i = 0; $i < sizeof($matches['filename']); $i++) {
            $file = $matches['filename'][$i];
            $files_to_upload[] = $file;
        }
    }

    unset($matches);
    $matches = array();
    // Old image regular expression:   /\[\[[iI][mM][aA][gG][eE]:(?P<filename>[^|\]]*)[^\]]*?/
    if (preg_match_all("/\[\[[iI][mM][aA][gG][eE]:(?P<filename>[^\|!@\$&\*\?:\[\]\"<>'`=\{\}\\/,;]*)[^\]]*\]\]/", $text, $matches)) {
        for ($i = 0; $i < sizeof($matches['filename']); $i++) {
            $file = $matches['filename'][$i];
            $files_to_upload[] = $file;
        }
    }

    unset($matches);
    $matches = array();
    // Old file regular expression:   /\[\[[fF][iI][lL][eE]:(?P<filename>[^|\]]*)[^\]]*?/
    if (preg_match_all("/\[\[[fF][iI][lL][eE]:(?P<filename>[^\|!@\$&\*\?:\[\]\"<>'`=\{\}\\/,;]*)[^\]]*\]\]/", $text, $matches)) {
        for ($i = 0; $i < sizeof($matches['filename']); $i++) {
            $file = $matches['filename'][$i];
            $files_to_upload[] = $file;
        }
    }

    return $files_to_upload;
}

function irods_files_post(&$text) {
    $files_to_upload = array();
    $matches = array();
    if (preg_match_all('/\{\{#[iI][rR][oO][dD][sS]:([^|!@$&*?:"<>\'`=,;]*)[^\}]*\}\}/', $text, $matches)) {
        for ($i = 0; $i < sizeof($matches[1]); $i++) {
            $file = $matches[1][$i];
            $files_to_upload[] = $file;
        }
    }

    return $files_to_upload;
}

function get_files_based_on_alias($param_alias) {
    $IrodsAccount = getAccount();

    // look for the mediaWiki Alias
    $flds = array("COL_DATA_NAME" => NULL, "COL_D_DATA_ID" => NULL,
        "COL_DATA_TYPE_NAME" => NULL, "COL_D_RESC_NAME" => NULL,
        "COL_DATA_SIZE" => NULL, "COL_D_OWNER_NAME" => NULL, "COL_D_OWNER_ZONE" => NULL,
        "COL_D_CREATE_TIME" => NULL, "COL_D_MODIFY_TIME" => NULL,
        "COL_COLL_NAME" => NULL, "COL_D_COMMENTS" => NULL);
    $select = new RODSGenQueSelFlds(array_keys($flds), array_values($flds));
    $condition = new RODSGenQueConds();
    $condition->add('COL_META_DATA_ATTR_NAME', 'LIKE', 'wiki_alias');
    $condition->add('COL_META_DATA_ATTR_VALUE', 'LIKE', $param_alias);
    $condition->add('COL_COLL_NAME', 'not like', '%/trash/%');
    $condition->add('COL_COLL_NAME', 'like', '/%'); // recursive search

    $conn = RODSConnManager::getConn($IrodsAccount);
    $results = $conn->query($select, $condition, 0, -1);
    RODSConnManager::releaseConn($conn);

    $vals = $results->getValues();

    if (sizeof($vals) == 0)
        return false;

    return $vals;
}

?>
