<?php
/**
 * Copyright 2013 Drexel University
 */

class SpecialIrodsUserManagement extends SpecialPage {

    function __construct() {
        require_once (dirname(__FILE__) . "/RodsWiki.functions.php");
        parent::__construct('SpecialIrodsUserManagement');
    }

    function execute($par) {

        global $wgOut, $wgUser, $wgDBprefix, $wgScriptPath, $configIrodsHost, $configIrodsPort;
        $this->setHeaders();

        if ($wgUser->mId == 0)
            die("Access restricted");

        $dbr = wfGetDB(DB_SLAVE);
        $dbw = wfGetDB(DB_MASTER);
        $dbw->query('
            CREATE TABLE IF NOT EXISTS ' . $wgDBprefix . 'irods_users (
            wiki_user_id int(5) NOT NULL PRIMARY KEY,
            irods_user_name varchar(64) NOT NULL,
            irods_user_password varchar(64) NOT NULL,
            irods_user_root_path varchar(128) NOT NULL,
            irods_user_resource varchar(128) NOT NULL,
            irods_user_zone varchar(64) NOT NULL,
            irods_custom_account_active tinyint(1) NOT NULL DEFAULT \'0\'
        );');

        $user_info = array();
        $user_info['wiki_user_id'] = $wgUser->mId;

        // add iRODS user
        if (isset($_POST['setting'])) {
            $user_info['irods_user_name'] = $_POST['irods_user_name'];
            $user_info['irods_user_password'] = $_POST['irods_user_password'];
            $user_info['irods_user_root_path'] = $_POST['irods_user_root_path'];
            $user_info['irods_user_resource'] = $_POST['irods_user_resource'];
            $user_info['irods_user_zone'] = $_POST['irods_user_zone'];
            $user_info['irods_custom_account_active'] = $_POST['setting'];

            if ($this->check_connection($configIrodsHost, $configIrodsPort, $user_info['irods_user_name'], $user_info['irods_user_password'])) {
                $dbw->delete("irods_users", array("wiki_user_id" => $wgUser->mId));
                $dbw->insert("irods_users", $user_info);
                $wgOut->addHtml('<p style="color: green; font-size: 18px; font-weight: bold">Account credentials updated</p>');
            } else {
                $wgOut->addHtml('<p style="color: red; font-size: 18px; font-weight: bold">Invalid account credentials</p>');
            }
        } else {
            $row_curr_user = $dbr->selectRow(array("irods_users"), array("irods_user_name, irods_user_password, irods_user_root_path, irods_user_resource, irods_user_zone, irods_custom_account_active"), array("wiki_user_id" => $wgUser->mId));
            if ($row_curr_user === false) {
                $user_info['irods_user_name'] = '';
                $user_info['irods_user_password'] = '';
                $user_info['irods_user_root_path'] = '';
                $user_info['irods_user_resource'] = '';
                $user_info['irods_user_zone'] = '';
                $user_info['irods_custom_account_active'] = '';
            } else {
                $user_info['irods_user_name'] = $row_curr_user->irods_user_name;
                $user_info['irods_user_password'] = $row_curr_user->irods_user_password;
                $user_info['irods_user_root_path'] = $row_curr_user->irods_user_root_path;
                $user_info['irods_user_resource'] = $row_curr_user->irods_user_resource;
                $user_info['irods_user_zone'] = $row_curr_user->irods_user_zone;
                $user_info['irods_custom_account_active'] = $row_curr_user->irods_custom_account_active;
            }
        }

        $wgOut->addWikiText("== User rights ==");
        $wgOut->addWikiText("Current MediaWiki user: " . $wgUser->mName);
        $wgOut->addWikiText("=== Linked iRODS Account ===");

        $wgOut->addHTML("<form method=\"POST\" id=\"form_change_user\" action=\"http://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]\">");
        $wgOut->addHTML("<input type=\"radio\" value=\"0\" name=\"setting\" id=\"setting_def\"");
        if ($user_info['irods_custom_account_active'] == 0) {
            $wgOut->addHTML(" checked");
        }
        $wgOut->addHTML("> Use a the default iRODS Account<br>");
        $wgOut->addHTML("<input type=\"radio\" value=\"1\" name=\"setting\" id=\"setting_cus\"");
        if ($user_info['irods_custom_account_active'] != 0) {
            $wgOut->addHTML(" checked");
        }
        $wgOut->addHTML("> Use a custom iRODS Account");
        $wgOut->addHTML("<table id='custom_user'");
        if ($user_info['irods_custom_account_active'] == 0) {
            $wgOut->addHTML(" style='display:none'");
        }
        $wgOut->addHTML(">");
        $wgOut->addHTML("<tr><td>User name:</td> <td><input type=\"text\" value=\"" . $user_info['irods_user_name'] . "\" name=\"irods_user_name\" id=\"irods_user_name\"></td></tr>");
        $wgOut->addHTML("<tr><td>Password:</td> <td><input type=\"password\" value=\"" . $user_info['irods_user_password'] . "\" name=\"irods_user_password\" id=\"irods_user_password\"></td></tr>");
        $wgOut->addHTML("<tr><td>Root path:</td> <td><input type=\"text\" value=\"" . $user_info['irods_user_root_path'] . "\" name=\"irods_user_root_path\" id=\"irods_user_root_path\"></td></tr>");
        $wgOut->addHTML("<tr><td>Resource:</td> <td><input type=\"text\" value=\"" . $user_info['irods_user_resource'] . "\" name=\"irods_user_resource\" id=\"irods_user_resource\"></td></tr>");
        $wgOut->addHTML("<tr><td>Zone:</td> <td><input type=\"text\" value=\"" . $user_info['irods_user_zone'] . "\" name=\"irods_user_zone\" id=\"irods_user_zone\"></td></tr>");
        $wgOut->addHTML("</table>");
        $wgOut->addHTML("<br/><input type=\"submit\" value=\"Save iRODS account\">");
        $wgOut->addHTML("</form>");
        $wgOut->addHTML("<script src=\"" . $wgScriptPath . "/extensions/RodsWiki/SpecialUserManage.js\"></script>");
    }

    static function check_connection($configIrodsHost, $configIrodsPort, $configIrodsUser, $configIrodsPass) {
        $IrodsAccount = new RODSAccount($configIrodsHost, $configIrodsPort, $configIrodsUser, $configIrodsPass);
        try {
            $IrodsAccount->getUserInfo();
            return true;
        } catch (Exception $e) {
            return false;
        }
    }
}

?>
